/*
 * Persistent Systems Private Limited, Pune, India (website - http://www.pspl.co.in)
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that this
 * copyright notice and the following paragraph appears in all
 * copies of this software.
 *
 * DISCLAIMER OF WARRANTY.
 * -----------------------
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
 * IN NO EVENT SHALL THE RIGHTHOLDER BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */
#include "XPathHelper.h"
#include "InternalConstants.h"
#include <string.h>
#include <iostream>
using namespace std;

//#define DEBUG_MODE 1

const char VOTABLE_ROOT[] = "/VOTABLE";

XPathHelper::XPathHelper(void)
{
	// do nothing
}

/*
* Get subtree from the whole VOTable document.
*/
int XPathHelper::getResourceTree(Resource &r, const char * filename, const char * xpath, int *status)
{
	
#ifdef  DEBUG_MODE
				cout << "File is " << filename << endl;
#endif
	
#if !defined(XALAN_NO_NAMESPACES)
	using std::cerr;
	using std::cout;
	using std::endl;
#endif

	
	try
		{
			XMLPlatformUtils::Initialize();

			XPathEvaluator::initialize();

			{							
				int result = VOERROR;			

				int		theResultInt = 0;
				
				char xpathExpr[MAX_XPATH_EXPR_LEN];

				strcpy(xpathExpr, VOTABLE_ROOT);
				strcat(xpathExpr, xpath);
				const XObjectPtr xPtr(NULL);


				// Initialize the XalanSourceTree subsystem...
				XalanSourceTreeInit		theSourceTreeInit;

				// We'll use these to parse the XML file.
				XalanSourceTreeDOMSupport		theDOMSupport;
				XalanSourceTreeParserLiaison	theLiaison(theDOMSupport);

				// Hook the two together...
				theDOMSupport.setParserLiaison(&theLiaison);

				// Create an input source that represents a local file...
				const LocalFileInputSource	theInputSource(c_wstr(XalanDOMString(filename)));

				// Parse the document...
				XalanDocument* const	theDocument =
						theLiaison.parseXMLStream(theInputSource);
				assert(theDocument != 0);

				XPathEvaluator	theEvaluator;

				// OK, let's find the context node...
				XalanNode* const	theContextNode =
						theEvaluator.selectSingleNode(
							theDOMSupport,
							theDocument,
							XalanDOMString("/").c_str(),
							theDocument->getDocumentElement());

				if (theContextNode == 0)
				{
					cerr << "Warning -- No nodes matched the location path \""
						 << "/"
						 << "\"."
						 << endl
						 << "Execution cannot continue..."
						 << endl
						 << endl;

					//XPathEvaluator::terminate();

					//XMLPlatformUtils::Terminate();
				}
				else
				{
					// OK, let's evaluate the expression...
					XObjectPtr *xPtr =  new XObjectPtr(
						theEvaluator.evaluate(
								theDOMSupport,
								theContextNode,
								XalanDOMString(xpathExpr).c_str(),
								theDocument->getDocumentElement()));

					assert((*xPtr).null() == false);
					
					extractResourceInfo(*xPtr, r, status);


#ifdef  DEBUG_MODE
					cout << "Got RESOURCE from the file." << endl;
#endif

					delete xPtr;
			

				
				}
			}
		

			XPathEvaluator::terminate();

			XMLPlatformUtils::Terminate();
		}
		
		catch(...)
		{
			*status = PARSING_ERROR;
			cerr << "Exception caught !!" << endl;
		}
	
	
	cout.flush();
	//return *(new XObjectPtr());
	return *status;
	
}


/*
* Get subtree from the whole VOTable document.
*/
int XPathHelper::getVTableTree(VTable &v, const char * filename, const char * xpath, int *status)
{

#ifdef  DEBUG_MODE
	cout << "File is " << filename << endl;
#endif
	
#if !defined(XALAN_NO_NAMESPACES)
	using std::cerr;
	using std::cout;
	using std::endl;
#endif

	
	try
		{
			XMLPlatformUtils::Initialize();

			XPathEvaluator::initialize();

			{				
			
				int result = VOERROR;			

				int		theResultInt = 0;
				
				char xpathExpr[MAX_XPATH_EXPR_LEN];

				strcpy(xpathExpr, VOTABLE_ROOT);
				strcat(xpathExpr, xpath);
				const XObjectPtr xPtr(NULL);
				

				// Initialize the XalanSourceTree subsystem...
				XalanSourceTreeInit		theSourceTreeInit;

				// We'll use these to parse the XML file.
				XalanSourceTreeDOMSupport		theDOMSupport;
				XalanSourceTreeParserLiaison	theLiaison(theDOMSupport);

				// Hook the two together...
				theDOMSupport.setParserLiaison(&theLiaison);

				// Create an input source that represents a local file...
				const LocalFileInputSource	theInputSource(c_wstr(XalanDOMString(filename)));

				// Parse the document...
				XalanDocument* const	theDocument =
						theLiaison.parseXMLStream(theInputSource);
				assert(theDocument != 0);

				XPathEvaluator	theEvaluator;

				// OK, let's find the context node...
				XalanNode* const	theContextNode =
						theEvaluator.selectSingleNode(
							theDOMSupport,
							theDocument,
							XalanDOMString("/").c_str(),
							theDocument->getDocumentElement());

				if (theContextNode == 0)
				{
					cerr << "Warning -- No nodes matched the location path \""
						 << "/"
						 << "\"."
						 << endl
						 << "Execution cannot continue..."
						 << endl
						 << endl;

					//XPathEvaluator::terminate();

					//XMLPlatformUtils::Terminate();
				}
				else
				{
					// OK, let's evaluate the expression...
					XObjectPtr *xPtr =  new XObjectPtr(
						theEvaluator.evaluate(
								theDOMSupport,
								theContextNode,
								XalanDOMString(xpathExpr).c_str(),
								theDocument->getDocumentElement()));

					assert((*xPtr).null() == false);
				
					extractTableInfo(*xPtr, v, status);
				
					delete xPtr;				
				}
			}
		

			XPathEvaluator::terminate();

			XMLPlatformUtils::Terminate();
		}
		
		catch(...)
		{
			*status = PARSING_ERROR;
			cerr << "Exception caught !!" << endl;
		}
	
	
	cout.flush();
	//return *(new XObjectPtr());
	return *status;
	
}

/*
* Extract information into VTable from the DOM tree.
*/
int XPathHelper::extractResourceInfo(XObjectPtr  xObj, Resource &r, int *status)
{
	int result = SUCCESS;

	
	// check if the node is an element.
	int nodeType = xObj->getType () ;
	
	//std::cout << "Node Type is " << nodeType << endl;

	if (nodeType == XObject::eTypeNodeSet)
	{
		const NodeRefListBase& nodeset = xObj->nodeset();
		size_t len = nodeset.getLength();

#ifdef  DEBUG_MODE
		cout << "Number of Resource nodes are " << len << endl;
#endif

		if (0 == len)
		{
			result = XPATH_RESOURCE_ERROR;
		}

		for (size_t i=0; i<len; i++)
		{
			XalanNode* const	node = nodeset.item(i);
			XalanDOMString		str;

			const int theType = node->getNodeType();
			//std::cout << "Node Type is " << theType << endl;

			
			if (theType == XalanNode::ELEMENT_NODE) 
			{
				
				str = node->getNodeName();
#ifdef  DEBUG_MODE
				cout << "Node name is " << str << endl;
#endif
				// check if element is table
				if (str.compare(ELE_RESOURCE) == 0)
				{	
					try 
					{
						getResource(node, r, &result);								
					} 
					catch (bad_alloc ex)
					{
						result = INSUFFICIENT_MEMORY_ERROR;
					}
				
					break;
				} 
				else
				{

#ifdef  DEBUG_MODE
					cout << "Node is not RESOURCE, returning ERROR ."<< endl;
#endif

					result = XPATH_RESOURCE_ERROR;
				}
			}
			else
			{
				result = XPATH_RESOURCE_ERROR;
			}			
		}

	} 
	else 
	{
		result = XPATH_RESOURCE_ERROR;
#ifdef  DEBUG_MODE
		cout << "There is no node type in this Object.";
#endif
	}
	
	*status = result;
	return *status;

}

/*
* Extract information into VTable from the DOM tree.
*/
int XPathHelper::extractTableInfo(XObjectPtr xObj, VTable &vTable, int *status)
{
	int result = SUCCESS;

	
	
	// check if the node is an element.
	int nodeType = xObj->getType () ;
	
	//std::cout << "Node Type is " << nodeType << endl;

	if (nodeType == XObject::eTypeNodeSet)
	{
		const NodeRefListBase& nodeset = xObj->nodeset();
		size_t len = nodeset.getLength();

#ifdef  DEBUG_MODE
		cout << "Number of TABLE nodes are " << len << endl;
#endif
		if (0 == len)
		{
			result = XPATH_VTABLE_ERROR;
		}

		for (size_t i=0; i<len; i++)
		{
			XalanNode* const	node = nodeset.item(i);
			XalanDOMString		str;

			const int theType = node->getNodeType();
			//std::cout << "Node Type is " << theType << endl;

			
			if (theType == XalanNode::ELEMENT_NODE) 
			{
				
				str = node->getNodeName();
				//cout << "Node name is " << str << endl;
				// check if element is table
				if (str.compare(ELE_TABLE) == 0)
				{	
					try 
					{
						getTable(node, vTable,  &result);								
					}
					catch (bad_alloc ex)
					{
						result = INSUFFICIENT_MEMORY_ERROR;
					}
					break;
				} 
				else
				{
#ifdef  DEBUG_MODE
					cout << "Node is not TABLE, returning ERROR ."<< endl;
#endif

					result = XPATH_VTABLE_ERROR;
				}
			}
			else
			{
				result = XPATH_VTABLE_ERROR;
			}			
		}

	} 
	else 
	{
		result = XPATH_VTABLE_ERROR;
#ifdef  DEBUG_MODE
		cout << "There is no node type in this Object.";
#endif
	}
	
	*status = result;
	return *status;

}

/*
* Process the child nodes of Resource such as Param, Link, Description etc.
* and fill them up.
*/
int XPathHelper::getResource(const XalanNode * node, Resource &r, int * status)
{

	// get attributes
	getResourceAttributes(node, r, status);

	XalanNode * const childNode = node->getFirstChild();

	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "Resource has no child nodes." << endl;
#endif
		*status = SUCCESS;
		return *status;
	}

	//cout << "First node name is " << childNode->getNodeName() << endl;
	//cout << "First node type is " << childNode->getNodeType() << endl;

	//XalanNode * const newSiblingNode = childNode;
	const XalanNode * siblingNode;
	int nodes = 0, eleNodes = 0;
	XalanDOMString	name;
	int result = 0;
	char *desc = NULL;

	siblingNode = childNode;
	vector<Info> infos;
	vector<Coosys> coosysList;
	vector<Param> params;
	vector<Link> links;
	vector<VTable> vtables;
	vector<Resource> resources;

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
#ifdef  DEBUG_MODE
			cout << "Node name is " << siblingNode->getNodeName() << endl;
#endif

			// check if node is info node
			if (name.compare(ELE_INFO) == 0) {
				Info i;

				getInfo(siblingNode, i, &result);
				infos.push_back(i);
			} else if (name.compare(ELE_COOSYS) == 0) { // check if node is coosys node
				Coosys c;
				getCoosys(siblingNode, c, &result);

				coosysList.push_back(c);
			} else if (name.compare(ELE_PARAM) == 0) { // check if node is param node
				Param p;
				getFieldParam(siblingNode, &p, false, &result);
				params.push_back(p);
			} else if (name.compare(ELE_DESC) == 0) { // check if node is description
				desc = getDescription(siblingNode, &result);
			} else if (name.compare(ELE_LINK) == 0) { // check if node is link node
				Link l;
				getLink(siblingNode,l, &result);

				links.push_back(l);
			} else if (name.compare(ELE_TABLE) == 0) { // check if node is table node
				VTable v;
				getTable(siblingNode, v, &result);
				vtables.push_back(v);
			} else if (name.compare(ELE_RESOURCE) == 0)	{
				Resource r;
				getResource(siblingNode,r, &result);
				resources.push_back(r);
			}
		}
		//XalanNode * const newSiblingNode = siblingNode->getNextSibling();
		siblingNode = siblingNode->getNextSibling();

	}

	r.setCoosystems(coosysList, &result);
	r.setInfos (infos, &result);
	r.setParams (params, &result);
	r.setLinks (links, &result);
	r.setTables (vtables, &result);
	r.setResources (resources, &result);
	r.setDesc (desc, &result);

	*status = SUCCESS;
	return SUCCESS;

}


/*
* Process the child nodes of Table such as Field, Link, Description etc.
* and fill them up.
*/
int XPathHelper::getTable(const XalanNode* node, VTable &v, int * status)
{

	getTableAttributes(node, v, status);

	const XalanNode * childNode = node->getFirstChild();

	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "Table has no child nodes." << endl;
#endif
		*status = VOERROR;
		return *status;
	}

	//cout << "First node name is " << childNode->getNodeName() << endl;
	//cout << "First node type is " << childNode->getNodeType() << endl;

	const XalanNode * siblingNode;
	//int nodes = 0, eleNodes = 0;
	XalanDOMString	name;
	int result = 0;
	char *desc = NULL;

	siblingNode = childNode;
	vector<Field> fields;
	vector<Link> links;
	TableMetaData tmd;
	TableData td;
	bool isField = true;


	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{

			name = siblingNode->getNodeName();
#ifdef  DEBUG_MODE
			cout << "Node name is " << siblingNode->getNodeName() << endl;
#endif


			// check if node is field node
			if (name.compare(ELE_FIELD) == 0) {
				Field f;
				getFieldParam(siblingNode,&f,isField,&result);
				fields.push_back(f);
			} else if (name.compare(ELE_LINK) == 0) { // check if node is link node
				Link l;

				getLink(siblingNode, l, &result);

				links.push_back(l);
			} else if (name.compare(ELE_DESC) == 0) { // check if node is description
				desc = getDescription(siblingNode, &result);
			} else if (name.compare(ELE_DATA) == 0)	{
				// get data
				getData(siblingNode, td, fields, &result);

			}
		
		//	eleNodes++;
		}
		
		siblingNode = siblingNode->getNextSibling();
	//	nodes++;

	}
	//cout << "Total no. is nodes is " << nodes << endl;
	tmd.setDesciption(desc, &result);
	tmd.setFields(fields, &result);
	tmd.setLinks(links, &result);
	v.setMetaData(tmd, &result);
	v.setData(td, &result);

#ifdef  DEBUG_MODE
	cout << "Filled the Table node." << endl;
#endif

	*status = SUCCESS;
	return *status;

}

/*
* Convert from XalanDOMString to char *.
* All the classes of VTable internally make use of char * 
* and not XalanDOMChar * to store strings. Need an explicit 
* conversion from 'short' to 'char'.
*/
char * XPathHelper::getCharString(XalanDOMString str)
{
	if (str.size() == 0) 
	{ 
		return NULL;
	}

	int length = str.size();
	char * charStr;
	//cout << "Length of " << str << " is = " << length << endl;

	//try {
		charStr = new char[length + 1];
	//} 
	//catch (bad_alloc ex)
	//{
//#ifdef  DEBUG_MODE
//	cout << "Cannot allocate required memory." << endl;
//#endif
		// Cannot allocate memory, exit ?
	//	return NULL;
	//}
	charStr[length] = 0;

	const XalanDOMChar * xdc = str.c_str();
	int i = 0;
	while (i < length)
	{
		// explicit conversion from 'short' to 'char'.
		charStr[i] = (char) xdc[i];
		i++;
	}
	charStr[length] = 0;

	//cout << "The returned string is = " << charStr << endl;

	return charStr;

}

/*
* Convert from XalanDOMString to short *.
* Used for storing Unicode.
*/
unsigned short * XPathHelper::getShortString(XalanDOMString str, int &size)
{
	size = 0; 
	if (str.size() == 0) 
	{ 
		
		return NULL;
	}

	int length = str.size();
	unsigned short * charStr;
#ifdef  DEBUG_MODE
	cout << "Length is = " << length << endl;
#endif

	//try 
	{
		charStr = new unsigned short[length + 1];
	}
	//catch (bad_alloc ex)
	//{
//#ifdef  DEBUG_MODE
//	cout << "Cannot allocate required memory." << endl;
//#endif
//		size = 0;
//		return NULL;
//	}
	charStr[length] = 0;

	const XalanDOMChar * xdc = str.c_str();
	/*int i = 0;
	int j = 0;
	while (i < length)
	{
		charStr[j] = xdc[i];
		charStr[j] = charStr[j] << 8;
		if (i + 1 < length)
		{
			charStr[j] += xdc[i + 1];
		}
		//cout << "charStr[" << j << "] = " << charStr[j];
		i += 2;
		j++;
	}
	charStr[j] = 0;
	cout << " j = " << j << "  and i = " << i << endl;
	cout << "size = " << (length/2 + 2) << endl;
	*/
	

	int i = 0;
	while (i < length)
	{
#ifdef  DEBUG_MODE
		cout << ((char) xdc[i]) << " ";
#endif
		
		charStr[i] = xdc[i];
		i++;
	}

	//cout << "The returned string is = " << charStr << endl;

	size = length + 1;
	//cout << endl << "New Length of " << str << " is = " << j+1 << endl;

	return charStr;

}


/*
* Get dataype of the field.
*/
int getFieldDatatype(char * type)
{
	
	for (int i = 1; i <= NUM_OF_DATATYPES; i++)
	{
		if (strcmp(type, FIELD_DATATYPE[i]) == 0)
		{
	
			return i;
		}
	}
	return 0;

}


/*
* Get system of the coosys.
*/
int getCoosysSystem(char * type)
{
	
	for (int i = 1; i <= NUM_OF_COOSYS_SYSTEMS; i++)
	{
		if (strcmp(type, COOSYS_SYSTEM[i]) == 0)
		{
			return i;
		}
	}
	return 0;

}

/*
* Get type of the field.
*/
int getFieldType(char * type)
{
	for (int i = 1; i <= NUM_OF_FIELD_TYPES; i++)
	{
		if (strcmp(type, FIELD_TYPE[i]) == 0)
		{
			return i;
		}
	}
	return 0;

}

/*
* Get content-role of the link.
*/
int getContentRole(char * role)
{
	for (int i = 1; i <= NUM_OF_CONTENT_ROLES; i++)
	{
		if (strcmp(role, CONTENT_ROLE[i]) == 0)
		{
			return i;
		}
	}
	return 0;

}

/*
* Get attributes of 'Resource'.
*/
int XPathHelper::getResourceAttributes(const XalanNode *node, Resource &r, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	
	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Resource attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_ID) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			r.setID(charValue, &result);
			
		} 
		else if (name.compare(ATTR_NAME) == 0)
		{
			// set ID
			charValue = getCharString(value);
			r.setName(charValue, &result);
			
		}
		else if (name.compare(ATTR_TYPE) == 0)
		{
			// set ID
			charValue = getCharString(value);
			resource_type t = results;
			if (strcmp(charValue, RESOURCE_META_TYPE) == 0)
			{
				t = meta;
			}
			r.setType(t, &result);
			delete[] charValue;
			
		}
	}

#ifdef  DEBUG_MODE
		cout << "Filled Resource node." << endl;
#endif
	*status = SUCCESS;
	
	return *status;
}



/*
* Get attributes of table.
*/
int XPathHelper::getTableAttributes(const XalanNode *node, VTable &v, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	
	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Table attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_ID) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			v.setID(charValue, &result);
			
		} 
		else if (name.compare(ATTR_NAME) == 0)
		{
			// set ID
			charValue = getCharString(value);
			v.setName(charValue, &result);
			
		}
		else if (name.compare(ATTR_REF) == 0)
		{
			// set ID
			charValue = getCharString(value);
			v.setRef(charValue, &result);
			
		}
	}
	*status = SUCCESS;
	return *status;
}

/*
* Get attributes of Column (Cell).
*/
int XPathHelper::getColumnAttributes(const XalanNode *node, Column &c, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	
	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Column attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_REF) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			c.setRef(charValue, &result);
			
		} 
		
	}
	return SUCCESS;
}


/*
* Get attributes of option.
*/
int XPathHelper::getOptionAttributes(const XalanNode *node, Option &o, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	
	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Option attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_NAME) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			o.setName(charValue, &result);
			
		} 
		else if (name.compare(ATTR_VALUE) == 0)
		{
			// set ID
			charValue = getCharString(value);
			o.setValue(charValue, &result);
			
		}
		
	}

	*status = SUCCESS;
	return SUCCESS;
}

/*
* Get attributes of Info.
*/
int XPathHelper::getInfoAttributes(const XalanNode *node, Info &info, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;

	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Info attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_NAME) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			info.setName(charValue, &result);
			
		} 
		else if (name.compare(ATTR_VALUE) == 0)
		{
			// set ID
			charValue = getCharString(value);
			info.setValue(charValue, &result);
			
		}
		else if (name.compare(ATTR_ID) == 0)
		{
			// set ID
			charValue = getCharString(value);
			info.setID(charValue, &result);
			
		}
		
	}
	*status = SUCCESS;
	return SUCCESS;
}

/*
* Get attributes of Coosys.
*/
int XPathHelper::getCoosysAttributes(const XalanNode *node, Coosys &c, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;


	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Coosys attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_ID) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			c.setID(charValue, &result);
			
		} 
		else if (name.compare(ATTR_EQUINOX) == 0)
		{
			// set equinox
			charValue = getCharString(value);
			c.setEquinox(charValue, &result);
			
		}
		else if (name.compare(ATTR_EPOCH) == 0)
		{
			// set epoch
			charValue = getCharString(value);
			c.setEpoch(charValue, &result);
			
		}
		else if (name.compare(ATTR_SYSTEM) == 0)
		{
			// set system
			charValue = getCharString(value);
			
			int sys = getCoosysSystem(charValue);

			delete[] charValue;

			coosys_system system;
			if (sys == 0)
			{
				system = eq_FK5;
			} 
			else 
			{
				switch (sys)
				{
				case eq_FK4 :
				{
					system = eq_FK4;
					break;
				}
				case eq_FK5 :
				{
					system = eq_FK5;
					break;
				}
				case ICRS :
				{
					system = ICRS;
					break;
				}
				case ecl_FK4 :
				{
					system = ecl_FK4;
					break;
				}
				case ecl_FK5 :
				{
					system = ecl_FK5;
					break;
				}
				case galactic :
				{
					system = galactic;
					break;
				}
				case supergalactic :
				{
					system = supergalactic;
					break;
				}
				case xy :
				{
					system = xy;
					break;
				}
				case barycentric :
				{
					system = barycentric;
					break;
				}
				case geo_app :
				{
					system = geo_app;
					break;
				}
				default :
				{
					system = eq_FK5;
					break;
				}
				} // end of switch

			}
		    c.setSystem(system, &result);		
		}
		
	}
	*status = SUCCESS;
	return SUCCESS;
}


/*
* Get attributes of range.
*/
Range XPathHelper::getRangeAttributes(const XalanNode *node, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	Range r;

	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Range attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;	
#endif
		
		
		if (name.compare(ATTR_VALUE) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			r.setValue(charValue, &result);
			
		} 
		else if (name.compare(ATTR_INCLUSIVE) == 0)
		{
			// set ID
			charValue = getCharString(value);
			bool inclusive = true;
			if (strcmp(charValue, NO) == 0)
			{
				inclusive = false;				
			} 
			delete[] charValue;
		
			r.setInclusiveFlag(inclusive, &result);
			
		}
		
	}
	*status = SUCCESS;
	return r;
}

/*
* Get attributes of table.
*/
int XPathHelper::getValuesAttributes(const XalanNode *node, Values &v, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	

	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Values attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_ID) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			v.setID(charValue, &result);
			
		} 
		else if (name.compare(ATTR_TYPE) == 0)
		{
			// set ID
			charValue = getCharString(value);
			values_type type;
			if (strcmp(charValue, VALUES_ACTUAL_TYPE) == 0) 
			{
				type = actual;
			} 
			else 
			{
				type = legal;
			}
			v.setType(type, &result);
			delete[] charValue;
			
		}
		else if (name.compare(ATTR_NULL) == 0)
		{
			// set ID
			charValue = getCharString(value);
			v.setNull(charValue, &result);
			
		}
		else if (name.compare(ATTR_INVALID) == 0)
		{
			// set ID
			charValue = getCharString(value);
			bool invalid;
			if (strcmp(charValue, YES) == 0) 
			{
				invalid = true;
			} 
			else 
			{
				invalid = false;
			}
			v.setInvalidFlag(invalid, &result);
			delete[] charValue;
			
		}
	}


	return SUCCESS;
}


/*
* Get attributes of field.
*/
int XPathHelper::getFieldParamAttributes(const XalanNode *node, FieldParam *fp, bool isField, int *status)
{
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;
	

	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Field attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_ID) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			(*fp).setID(charValue, &result);
			
		} 
		else if (name.compare(ATTR_UNIT) == 0)
		{
			// set unit
			charValue = getCharString(value);
			(*fp).setUnit(charValue, &result);
		}
		else if (name.compare(ATTR_DATATYPE) == 0)
		{
			// set datatype
			charValue = getCharString(value);
			int dt = getFieldDatatype(charValue);

			// delete here itself.
			delete[] charValue;
			field_datatype fdt;

			if (dt == 0)
			{
				//cout << "Field does not have dataype." << endl;
				//f.error = VOERROR;
				//result = VOERROR;
				fdt = datatype_not_specified;
			} else {
				
				switch (dt) {
					case BooleanType:  fdt = BooleanType; break;
					case BitType :  fdt = BitType; break; 
					case UnsignedByteType :  fdt = UnsignedByteType; break; 
					case ShortType :  fdt = ShortType; break; 
					case IntType :  fdt = IntType; break; 
					case LongType :  fdt = LongType; break; 
					case CharType :  fdt = CharType; break; 
					case UnicodeCharType :  fdt = UnicodeCharType; break; 
					case FloatType :  fdt = FloatType; break; 
					case DoubleType :  fdt = DoubleType; break; 
					case FloatComplexType :  fdt = FloatComplexType; break; 
					case DoubleComplexType :  fdt = DoubleComplexType; break; 
				}
				
			}
			(*fp).setDatatype(fdt, &result);
			
		}
		else if (name.compare(ATTR_PRECISION) == 0)
		{
			// set precision
			charValue = getCharString(value);
			(*fp).setPrecision(charValue, &result);
		}
		else if (name.compare(ATTR_WIDTH) == 0)
		{
			// set width
			charValue = getCharString(value);
			// convert string to 'int'.
			unsigned int width = atoi(charValue);
			(*fp).setWidth(width, &result);
			delete[] charValue;
		}
		else if (name.compare(ATTR_REF) == 0)
		{
			// set ref
			charValue = getCharString(value);
			(*fp).setRef(charValue, &result);
		}
		else if (name.compare(ATTR_NAME) == 0)
		{
			// set name
			charValue = getCharString(value);
			(*fp).setName(charValue, &result);
		}
		else if (name.compare(ATTR_UCD) == 0)
		{
			// set UCD
			charValue = getCharString(value);
			(*fp).setUCD(charValue, &result);
		}
		else if (name.compare(ATTR_ARRAYSIZE) == 0)
		{
			// set arraysize
			charValue = getCharString(value);
			(*fp).setArraySize(charValue, &result);
		} 
		else if (isField && name.compare(ATTR_TYPE) == 0)
		{
			// set type
			charValue = getCharString(value);
			int type = getFieldType(charValue);
			field_type ft;
			switch (type) {
				case type_not_specified:  ft = type_not_specified; break;
				case hidden:  ft = hidden; break;
				case no_query:  ft = no_query; break;
				case trigger:  ft = trigger; break;
			}	 
			delete[] charValue;

			(*fp).setType(ft, &result);
		}
		else if (!isField && name.compare(ATTR_VALUE) == 0)
		{
			// set arraysize
			charValue = getCharString(value);
			(*fp).setValue(charValue, &result);
		}

	}

	//Field f1 = new Field(f);

	return SUCCESS;
}

/*
* Fill in the 'Field'.s
*/
int XPathHelper::getFieldParam(const XalanNode *node, FieldParam *fp, bool isField, int *status)
{
	int result = 0;

	// get field attributes
	getFieldParamAttributes(node, fp, isField, &result);
	
	XalanDOMString	name;
	
	const XalanNode * childNode = node->getFirstChild();

	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "Field has no child nodes." << endl;
#endif
		return SUCCESS;
	}

	const XalanNode * siblingNode;
	
	vector <Link> links;
	vector <Values> values;
	Link l;
	Values v[2];
	siblingNode = childNode;
	int valueIndex = 0;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
#ifdef  DEBUG_MODE
			cout << "Node name is " << siblingNode->getNodeName() << endl;
#endif

			// check if node is description node
			if (name.compare(ELE_DESC) == 0) {
				(*fp).setDescription(getDescription(siblingNode, &result), &result) ;
			} else if (name.compare(ELE_LINK) == 0) { // check if node is link node
				Link l;
				getLink(siblingNode,l, &result);
				links.push_back(l);				
			} else if (name.compare(ELE_VALUES) == 0) { // check if node is Values
				// there should be only 2 'values' nodes.
				if (valueIndex < 2) 
				{
					getValues(siblingNode, v[valueIndex], &result);
					valueIndex++;
				}
			} 
		}
		siblingNode = siblingNode->getNextSibling();
	}
	
	(*fp).setValues(v, valueIndex, &result);
	(*fp).setLinks(links, &result);

//	Field f1 = new Field(f);
//	delete &f;

	return SUCCESS;
}

int XPathHelper::getLink(const XalanNode *node, Link &l, int *status)
{
	
	const XalanNamedNodeMap * xnnp = node->getAttributes();
	int numOfAttribs = xnnp->getLength();
	XalanDOMString name, value;
	char *charValue = NULL;
	int result = 0;

	//cout << "Number of attributes are = " << numOfAttribs << endl;

	// need to get attributes one by one
	for (int i = 0; i <  numOfAttribs; i++)
	{
		XalanNode * attribNode = xnnp->item(i);

		name = attribNode->getNodeName();
		value = attribNode->getNodeValue();

#ifdef  DEBUG_MODE
		cout << "Link attribute name is = " << name << endl;
		cout << "Attribute value is = " << value << endl;
#endif
		
		
		if (name.compare(ATTR_ID) == 0) 
		{
			// set ID
			charValue = getCharString(value);
			l.setID(charValue, &result); 			
		} 
		else if (name.compare(ATTR_CONTENT_ROLE) == 0)
		{
			charValue = getCharString(value);
			int role = getContentRole(charValue);
			content_role cr;
			switch (role) {
				case role_not_specified:  cr = role_not_specified; break;
				case query:  cr = query; break;
				case hints:  cr = hints; break;
				case doc:  cr = doc; break;
			}	
					
			l.setContentRole(cr, &result);
			delete[] charValue;
		}
		else if (name.compare(ATTR_CONTENT_TYPE) == 0)
		{
			charValue = getCharString(value);
			l.setContentType(charValue, &result);
		}
		else if (name.compare(ATTR_TITLE) == 0)
		{
			charValue = getCharString(value);
			l.setTitle(charValue, &result);
		}
		else if (name.compare(ATTR_VALUE) == 0)
		{
			charValue = getCharString(value);
			l.setValue(charValue, &result);
		}
		else if (name.compare(ATTR_HREF) == 0)
		{
			charValue = getCharString(value);
			l.setHRef(charValue, &result);
		}
		else if (name.compare(ATTR_GREF) == 0)
		{
			charValue = getCharString(value);
			l.setGRef(charValue, &result);
		}
		else if (name.compare(ATTR_ACTION) == 0)
		{
			charValue = getCharString(value);
			l.setAction(charValue, &result);
		}
	}

	// get the PC Data
	char *pcdata = NULL;
	const XalanNode * childNode = node->getFirstChild();
	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "LINK has no child nodes." << endl;		
#endif
	} 
	else 
	{
		pcdata = getCharString(childNode->getNodeValue());
#ifdef  DEBUG_MODE
		cout << "PCDATA in Link is " << pcdata << endl;
#endif
	}

	l.setPCData(pcdata, &result);
	

	*status = SUCCESS;
	return SUCCESS;
}

/*
* Get the description.
*/
char * XPathHelper::getDescription(const XalanNode *node, int * status)
{
	char * desc = NULL;

	const XalanNode * childNode = node->getFirstChild();

	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "DESC has no child nodes." << endl;
#endif
		return desc;
	} 
	else 
	{
		desc = getCharString(childNode->getNodeValue());
#ifdef  DEBUG_MODE
		cout << "Description is " << desc << endl;
#endif
	}

	return desc;
}

/*
* Extract information from Values node.
*/
int XPathHelper::getValues(const XalanNode *node, Values &v, int *status)
{
	int result = 0;
	// get attributes of Values node
	getValuesAttributes(node, v, &result);

	// fill child nodes of Values
	XalanDOMString	name;
	
	const XalanNode * childNode = node->getFirstChild();
	const XalanNode * siblingNode;
	
	Range *minimum = NULL;
	Range *maximum = NULL;
	vector<Option> optionList;
	siblingNode = childNode;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
			//cout << "Node name is " << siblingNode->getNodeName() << endl;

			// check if node is 'Option' node
			if (name.compare(ELE_OPTION) == 0) {
				Option o;
				getOption(siblingNode, o, &result);
				optionList.push_back(o) ;
			} else if (name.compare(ELE_MIN) == 0) { // check if node is MIN 
				minimum = (getRange(siblingNode,&result));				
			} else if (name.compare(ELE_MAX) == 0) { // check if node is MAX
				maximum = (getRange(siblingNode,&result));
			} 
		}
		siblingNode = siblingNode->getNextSibling();
	}
	v.setMinimun(minimum, &result);
	v.setMaximum(maximum, &result);
	v.setOptions(optionList, &result);

	return SUCCESS;


}

/*
* Extract information from 'Info' node.
*/
int XPathHelper::getInfo(const XalanNode *node, Info &i, int *status)
{
	int result = 0;
	// get attributes of Values node
	getInfoAttributes(node, i, &result);

	// get the PC Data
	char *pcdata = NULL;
	const XalanNode * childNode = node->getFirstChild();
	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "LINK has no child nodes." << endl;		
#endif
	} 
	else 
	{
		pcdata = getCharString(childNode->getNodeValue());
#ifdef  DEBUG_MODE
		cout << "PCDATA in Link is " << pcdata << endl;
#endif
	}

	i.setPCData(pcdata, &result);

	*status = SUCCESS;
	return SUCCESS;
}

/*
* Extract information from 'Coosys' node.
*/
int XPathHelper::getCoosys(const XalanNode *node, Coosys &c, int *status)
{
	int result = 0;
	// get attributes of Values node
	getCoosysAttributes(node, c, &result);

	// get the PC Data
	char *pcdata = NULL;
	const XalanNode * childNode = node->getFirstChild();
	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "Coosys has no child nodes." << endl;		
#endif
	} 
	else 
	{
		pcdata = getCharString(childNode->getNodeValue());
#ifdef  DEBUG_MODE
		cout << "PCDATA in Link is " << pcdata << endl;
#endif
	}

	c.setPCData(pcdata, &result);

	*status = SUCCESS;
	return SUCCESS;
}


/*
* Extract information from Option node.
*/
int XPathHelper::getOption(const XalanNode *node, Option &o, int *status)
{
	int result = 0;
	// get attributes of Option node
	getOptionAttributes(node, o, &result);
		
	XalanDOMString	name;
	
	const XalanNode * childNode = node->getFirstChild();
	const XalanNode * siblingNode;
	
	vector<Option> optionList;
	siblingNode = childNode;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
			//cout << "Node name is " << siblingNode->getNodeName() << endl;

			// check if node is 'Option' node
			if (name.compare(ELE_OPTION) == 0) {
				Option option;
				getOption(siblingNode, option, &result);
				optionList.push_back(option) ;
			} 
		}
		siblingNode = siblingNode->getNextSibling();
	}
	
	o.setOptions(optionList, &result);

	*status = SUCCESS;
	return SUCCESS;

}


/*
* Extract information from Range (Min or Max) node.
*/
Range * XPathHelper::getRange(const XalanNode *node, int *status)
{
	int result = 0;
	// get attributes of Option node
	Range *r = new Range(getRangeAttributes(node, &result));

	// get the PC Data
	char *pcdata = NULL;
	const XalanNode * childNode = node->getFirstChild();
	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "Range has no child nodes." << endl;		
#endif
	} 
	else 
	{
		pcdata = getCharString(childNode->getNodeValue());
#ifdef  DEBUG_MODE
		cout << "PCDATA in Link is " << pcdata << endl;
#endif
	}

	(*r).setPCData(pcdata, &result);
	
	return r;

}

/*
* Extract information from Option node.

Option XPathHelper::getOption(const XalanNode *node, int *status)
{
	int result = 0;
	// get attributes of Option node
	Option o = getOptionAttributes(node, &result);
		
	XalanDOMString	name;
	
	const XalanNode * childNode = node->getFirstChild();
	const XalanNode * siblingNode;
	
	vector<Option> optionList;
	siblingNode = childNode;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
			cout << "Node name is " << siblingNode->getNodeName() << endl;

			// check if node is 'Option' node
			if (name.compare(ELE_OPTION) == 0) {
				optionList.push_back(getOption(siblingNode, &result)) ;
			} 
		}
		siblingNode = siblingNode->getNextSibling();
	}
	
	o.setOptions(optionList, &result);

	return o;

}
*/


/*
* Extract information from 'Data' node.
*/
int XPathHelper::getData(const XalanNode *node, TableData &td, 
						 const vector<Field> &fields, int *status)
{
	int result = 0;
			
	XalanDOMString	name;

	vector<Row> rowList;
	
	const XalanNode * childNode = node->getFirstChild();
	const XalanNode * siblingNode;
		
	siblingNode = childNode;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
			//cout << "Node name is " << siblingNode->getNodeName() << endl;

			// check if node is 'Option' node
			if (name.compare(ELE_TABLEDATA) == 0) {
				 getRows(siblingNode, rowList, fields, &result);
			} 
			else 
			{
				*status = VOERROR;
				// there is no table data in this table
				// Current implemeation supports only TableData
				//cout << "Current implemeation supports only TableData." << endl;
				return VOERROR;

			}
		}
		siblingNode = siblingNode->getNextSibling();
	}
	td.setRows(rowList, &result);

	*status = SUCCESS;
	return SUCCESS;

}

/*
* Extract information from 'TableData' node.
*/
int XPathHelper::getRows(const XalanNode *node, vector<Row> &rowList, 
						 const vector<Field> &fields, int *status)
{
	int result = 0;
			
	XalanDOMString	name;
	
	const XalanNode * childNode = node->getFirstChild();
	const XalanNode * siblingNode;
		
	siblingNode = childNode;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		Row r;
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
			//cout << "Node name is " << siblingNode->getNodeName() << endl;

			// check if node is 'Option' node
			if (name.compare(ELE_TR) == 0) {
				getRow(siblingNode, r, fields, &result);
				rowList.push_back(r);
			} 
			
		}
		siblingNode = siblingNode->getNextSibling();
	}
	
	*status = SUCCESS;
	return SUCCESS;

}

/*
* Extract information from 'Row' node.
*/
int XPathHelper::getRow(const XalanNode *node, Row &row, 
						const vector<Field> &fields, int *status)
{
	int result = 0;
			
	XalanDOMString	name;
	
	const XalanNode * childNode = node->getFirstChild();
	const XalanNode * siblingNode;
		
	siblingNode = childNode;
	vector<Column> columnList;
	int columnNum = 0;
		

	//get all siblings (not using getChildNodes since it is not working) :-(
	while (siblingNode != NULL) 
	{
		Column c;
		// process only element nodes, not text nodes.
		// To do - find some way of avoiding text nodes.

		//cout << "Next node name is " << siblingNode->getNodeName() << endl;
		//cout << "Next node type is " << siblingNode->getNodeType() << endl;
		if (siblingNode->getNodeType() == XalanNode::ELEMENT_NODE)
		{
			name = siblingNode->getNodeName();
			//cout << "Node name is " << siblingNode->getNodeName() << endl;

			// check if node is 'TD' node
			if (name.compare(ELE_TD) == 0) {
				bool isUnicode = false;
				field_datatype d;
#ifdef  DEBUG_MODE
						cout << "Col Num  " << columnNum << endl;		
#endif

				if (columnNum < fields.size ()) 
				{
					Field f = fields[columnNum];

					f.getDatatype(d, &result);
#ifdef  DEBUG_MODE
						cout << "Field  " << columnNum  << " dt is " << d << endl;		
#endif

					if (d == UnicodeCharType)
					{
						isUnicode = true;
#ifdef  DEBUG_MODE
						cout << "Getting unicode data for column " << columnNum << endl;		
#endif
					}


				}
				getColumn(siblingNode, c, isUnicode, &result);
				columnList.push_back(c);
				columnNum++;
			} 
			
		}
		siblingNode = siblingNode->getNextSibling();
	
	}

	row.setColumns(columnList, &result);


	return SUCCESS;

}

/*
* Extract information from 'Column' node.
*/
int XPathHelper::getColumn(const XalanNode *node, Column &c, bool isUnicode, int *status)
{
	int result = 0;

	getColumnAttributes(node, c, &result);
			
	XalanDOMString	name;
		
	const XalanNode * childNode = node->getFirstChild();

	char * data = NULL;
	
	if (childNode == NULL)
	{
#ifdef  DEBUG_MODE
		cout << "TD has no child nodes." << endl;		
#endif
	} 
	else 
	{
		if (isUnicode)
		{
			int size = 0;
			unsigned short *unicodeData = getShortString(childNode->getNodeValue(), size);
			c.setUnicodeData(unicodeData, size, status);
		}
		else
		{
			data = getCharString(childNode->getNodeValue());
			c.setCharData(data, status);
#ifdef  DEBUG_MODE
		cout << "Data in TD is " << data << endl;
#endif
		}

	}
	

	return SUCCESS;
	

}









